@@ -18,10 +18,6 @@ use Bootloader::Core;
 
 C<< $res = Bootloader::Core->trim ($string); >>
 
-C<< $mountpoint = Bootloader::Core->Dev2MP ($device); >>
-
-C<< $device = Bootloader::Core->MP2Dev ($mountpoint); >>
-
 C<< $members_ref = Bootloader::Core->MD2Members ($md_device); >>
 
 C<< $md_dev = Bootloader::Core->Member2MD (string $md_member); >>
@@ -129,6 +125,8 @@ use strict;
 #constants
 
 my $headline = "# Modified by YaST2. Last modification on ";
+my $headline2 = "# THIS FILE WILL BE PARTIALLY OVERWRITTEN by perl-Bootloader";
+my $headline3 = "# Configure custom boot parameters for updated kernels in /etc/sysconfig/bootloader";
 
 # variables
 
@@ -156,10 +154,7 @@ sub l_debug {
     my $self = shift;
     my $message = shift;
  
-    push @{$self->{"log_records"}}, {
-	"message" => $message,
-	"level" => "debug",
-    };
+    $self->{"logger"}->debug($message);
 }
 
 =item
@@ -172,10 +167,7 @@ sub l_milestone {
     my $self = shift;
     my $message = shift;
  
-    push @{$self->{"log_records"}}, {
-	"message" => $message,
-	"level" => "milestone",
-    };
+    $self->{"logger"}->milestone($message);
 }
 
 =item
@@ -188,10 +180,7 @@ sub l_warning {
     my $self = shift;
     my $message = shift;
  
-    push @{$self->{"log_records"}}, {
-	"message" => $message,
-	"level" => "warning",
-    };
+    $self->{"logger"}->warning($message);
 }
 
 =item
@@ -204,65 +193,9 @@ sub l_error {
     my $self = shift;
     my $message = shift;
 
-    push @{$self->{"log_records"}}, {
-	"message" => $message,
-	"level" => "error",
-    };
+    $self->{"logger"}->error($message);
 }
 
-=item
-C<< $records_ref = Bootloader::Core->GetLogRecords (); >>
-
-
-
-
-=cut
-sub GetLogRecords {
-    my $self = shift;
-
-    my $ret = $self->{"log_records"};
-    $self->{"log_records"} = [];
-    return $ret;
-}
-
-=item
-C<< $mountpoint = Bootloader::Core->Dev2MP ($device); >>
-
-Gets the mount point where a specified device is mounted. As argument
-takes the device to examinate (string, eg. C<'/dev/hda1'>), returns the
-mountopint where the device is mounter (string, eg. C<'/boot'>), or undef
-if the device is not mounted (according to /etc/fstab).
-
-=cut
-
-# string Dev2MP (string device)
-sub Dev2MP {
-    my $self = shift;
-    my $dev = shift;
-
-    foreach my $mp ( keys %{$self->{"mountpoints"}}) {
-	return $mp if $self->{"mountpoints"}->{$mp} eq $dev;
-    }
-    return undef;
-}
-
-=item
-C<< $device = Bootloader::Core->MP2Dev ($mountpoint); >>
-
-Gets the device where filesystem mounted at a specified mountpoint is
-located. As argument takes the mount point (string, eg. C<'/boot'>), returns
-the device (string, eg. C<'/dev/hda1'>), or undef if there is no device mounted
-in the specified directory
-
-=cut
-
-# string MP2Dev (string mountpoint)
-sub MP2Dev {
-    my $self = shift;
-    my $mp = shift;
-
-    return $self->{"mountpoints"}{$mp} || undef;
-}
 
 =item
 C<< $members_ref = Bootloader::Core->MD2Members ($md_device); >>
@@ -296,11 +229,12 @@ sub Member2MD {
     my $self = shift;
     my $member = shift;
 
-    # reset 'each' iterator
-    my $dummy = keys %{$self->{"md_arrays"}};
+    my $find = undef;
 
     while ((my $md, my $mem_ref) = each (%{$self->{"md_arrays"}}))
     {
+      unless (defined $find)
+      {
 	foreach my $mem (@{$mem_ref})
 	{
 	    if ($mem eq $member)
@@ -309,9 +243,12 @@ sub Member2MD {
 		{
 		    $md = "/dev/" . $md;
 		}
-		return $md;
+		$member = $md;
+                $find = 1;
+                last;
 	    }
 	}
+      }
     }
     return $member;
 }
@@ -380,7 +317,7 @@ sub SplitDevPath {
 
     my $dev = $self->{"mountpoints"}{$mp};
     $path =~ s#^$mp/?#/#;
-    $self->l_debug ("Core::SplitDevPath: $orig was split to $dev + $path");
+    $self->l_milestone ("Core::SplitDevPath: $orig was split to $dev + $path");
     return ($dev, $path);
 }
 
@@ -400,10 +337,11 @@ sub GetExtendedPartition {
     my $extended_part_dev = undef;
 
     # Check for valid devices
-    unless ($part_dev =~ m/^\/dev\/[sh]d[a-z](\d+)?$/) {
-	$self->l_debug ("Core::GetExtendedPartition: Specified device
-	    $part_dev is not valid and cannot be used as base for detecting an
-	    extended partition on the same disk.");
+    unless (defined $part_dev and $part_dev =~ m/^\/dev\/[sh]d[a-z](\d+)?$/) {
+	$self->l_milestone ("Core::GetExtendedPartition: Specified device ".
+	    $part_dev||"undefined partdev".
+            " is not valid and cannot be used as base for detecting an
+            extended partition on the same disk.");
 	return $extended_part_dev;
     }
 
@@ -419,7 +357,6 @@ sub GetExtendedPartition {
 	if ($part_ref->[1] eq $disk_dev and
 	    $part_ref->[5] eq "`extended") {
 	    $extended_part_dev = $part_ref->[0];
-	    last;
 	}
     }
 
@@ -468,16 +405,7 @@ sub Unquote {
     my $self = shift;
     my $text = shift;
 
-    # remove leading blanks
-    if ($text =~ /^[ \t]*([^ \t].*)$/)
-    {
-	$text = $1;
-    }
-    # remove tailing blanks
-    if ($text =~ /^(.*[^ \t])[ \t]$/)
-    {
-	$text = $1;
-    }
+    $text = $self->trim($text);
     #now unquote
     if ($text =~ /^\"(.*)\"$/)
     {
@@ -504,10 +432,16 @@ sub MergeIfDefined {
     {
 	if (defined ($arg))
 	{
+          if ($ret eq "") #first defined entry
+          {
+            $ret = $arg;
+          }
+          else
+          {
 	    $ret = $ret . " " . $arg;
+          }
 	}
     }
-    $ret = substr ($ret, 1) if substr ($ret, 0, 1) eq " ";
     return $ret;
 }
 
@@ -700,12 +634,14 @@ sub MenuFileLineEmpty {
     my $self = shift;
     my $line = shift;
 
-    if (index ($line, $headline) >= 0)
+    if (index ($line, $headline) >= 0
+        or index ($line, $headline2) >= 0
+        or index ($line, $headline3) >= 0)
     {
 	# ignore the headline with modification date/time
 	return 1;
     }
-    elsif ($line =~ /^[ \t]*$/)
+    elsif ($line =~ /^\s*$/)
     {
 	# ignore empty line
 	return 1;
@@ -812,7 +748,7 @@ sub CreateMenuFileLines {
 
     use POSIX qw(strftime);
     my $date = strftime "%a %b %e %H:%M:%S %Z %Y", localtime;
-    my @ret = ($headline . $date);
+    my @ret = ($headline . $date."\n".$headline2."\n".$headline3."\n");
 
     foreach my $l (@parsed_lines) {
 	push @ret, @{$l->{"comment_before"}} if defined $l->{"comment_before"};
@@ -892,7 +828,7 @@ sub new {
 	    $loader->{$key} = $old->{$key};
 	}
     }
-    $loader->{"log_records"} = [];
+    $loader->{"logger"} = Bootloader::Logger::instance();
 
     bless ($loader);
     return $loader;
@@ -989,7 +925,7 @@ sub WriteFiles {
     {
 	if (! defined ($menu_files{$filename}))
 	{
-	    $self->l_debug ("Core::WriteFiles: Not writing $filename");
+	    $self->l_milestone ("Core::WriteFiles: Not writing $filename");
 	    next;
 	}
 
@@ -1009,38 +945,87 @@ sub WriteFiles {
 }
 
 =item
-C<< $original_name = Bootloader::Core->Comment2OriginalName ($comment); >>
+C<< $original_name = Bootloader::Core->CompleteSection($section_info); >>
 
-Gets the original name of the section from the comment. As argument, takes
-the comment, returns the original name (if found in the comment), or empty
-string (otherwise).
+Fill missing required keys of section.
+
+=cut
+
+sub CompleteSection($) {
+  my $self = shift;
+  my $info = shift;
+
+  unless (defined $info->{"__handled"})
+  {
+    $self->l_milestone("Core::CompleteSection: Add missing handled section");
+    if ($info->{"type"} eq "image"
+      || $info->{"type"} eq "xen")
+    {
+      $info->{"__handled"} = "auto";
+      $info->{"__handled"} = "all";
+    }
+    else
+    {
+      $info->{"__handled"} = "user";
+    }
+  }
+}
+
+
+=item
+C<< $original_name = Bootloader::Core->Comment2OriginalName ($section_ref,$comment); >>
+
+Gets the aditional information of the section from the comment. As argument, takes
+reference to section and the comment.
 
 =cut
 
 my $orig_name_comment="###Don't change this comment - YaST2 identifier: Original name: ";
+my $auto_comment_read="###Don't change this comment - YaST2 identifier: Original name: \(.*\) Handled by: YaST2 flavor: \(.*\)###";
+my $auto_comment_write="###Don't change this comment - YaST2 identifier: Original name: %s Handled by: YaST2 flavor: %s###";
+my $manual_comment_read="###Don't change this comment - YaST2 identifier: Original name: \(.*\) Handled by: user###";
+my $manual_comment_write="###Don't change this comment - YaST2 identifier: Original name: %s Handled by: user###";
 
 # string Comment2OriginalName (list<string> comment)
-sub Comment2OriginalName($) {
+sub Comment2OriginalName() {
     my $self = shift;
+    my $section = shift;
     my $comment_lines_ref = shift || [];
-    foreach (@{$comment_lines_ref}) {
-	return $1
-	    if m/${orig_name_comment}([^#]+?) *###/o;
+    my $ret = "";
+    foreach my $comment (@{$comment_lines_ref}) {
+	if( $comment =~ m/$orig_name_comment(.*)###/)
+        {
+          $section->{"original_name"} = $1;
+          $ret = $1;
+        }
+=cut prepared autosections
+        elsif ( $comment =~ $auto_comment_read )
+        {
+          $section->{"original_name"} = $1;
+          $section->{"__handled"} = "auto";
+          $section->{"__flavor"} = $2;
+        }
+        elsif ( $comment =~ $manual_comment_read )
+        {
+          $section->{"original_name"} = $1;
+          $section->{"__handled"} = "user";
+        }
+=cut
     }
-    return "";
+    return $ret;
 }
 
 =item
-C<< $line_ref = Bootloader::Core->UpdateSectionNameLine ($name, \%line, $original_name); >>
+C<< $line_ref = Bootloader::Core->UpdateSectionNameLine ($section, \%line, $original_name); >>
 
 Updates the 'section name line' so that it contains specified name and original name
-inside the comment. As arguments, takes the name (string), the line (hash reference)
+inside the comment. As arguments, takes the section reference (hash reference), the line (hash reference)
 and the original name (string). If original name is set to undef or empty string,
 it is not set in the comment. Returns the updated line reference.
 
 =cut
 
-# map<string,any> UpdateSectionNameLine (string name, map<string,any> line, string original_name)
+# map<string,any> UpdateSectionNameLine (map section, map<string,any> line, string original_name)
 sub UpdateSectionNameLine {
     my $self = shift;
     my $name = shift;
@@ -1051,9 +1036,32 @@ sub UpdateSectionNameLine {
     if (defined ($original_name) && $original_name ne "")
     {
 	my @comment_before = grep {
-	    ! m/^${orig_name_comment}/o;
+          my $line = $_;
+          my $ret = 1;
+          if ($line =~ m/$orig_name_comment/
+            || $line =~ m/$auto_comment_read/
+            || $line =~ m/$manual_comment_read/)
+          {
+            $ret = 0;
+          }
+          $ret;
 	}  @{$line_ref->{"comment_before"} || []};
-	push @comment_before, "${orig_name_comment}${original_name}###";
+        push @comment_before, $orig_name_comment.$original_name."###";
+=cut
+        if ( $section->{"__handled"} eq "auto" )
+        {
+          my $flavor = $section->{"__flavor"};
+          my $comment = sprintf($auto_comment_write, $original_name, $flavor);
+	  push @comment_before, $comment;
+          $self->l_milestone( "Core::UpdateSectionNameLine: section name: $name comment: $comment" );
+        }
+        elsif( $section->{"__handled"} eq "manual")
+        {
+          my $comment = sprintf($manual_comment_write, $original_name);
+	  push @comment_before, $comment;
+          $self->l_milestone( "Core::UpdateSectionNameLine: section name: $name comment: $comment" );
+        }
+=cut
 	$line_ref->{"comment_before"} = \@comment_before;
     }
     return $line_ref;
@@ -1095,7 +1103,7 @@ sub CreateRemovedDefaultLine {
 
     push @comment_before, "${remove_default_comment}";
 
-    $self->l_milestone("put removed default comment");
+    $self->l_milestone("Core::CreateRemovedDefaultLine: put removed default comment");
 		
     $line_ref->{"comment_before"} = \@comment_before;
 
@@ -1123,7 +1131,7 @@ sub Section2Info {
 	if ($key eq "label")
 	{
 	    $ret{"name"} = $line_ref->{"value"};
-	    my $on = $self->Comment2OriginalName ($line_ref->{"comment_before"});
+	    my $on = $self->Comment2OriginalName (\%ret,$line_ref->{"comment_before"});
 	    $ret{"original_name"} = $on if ($on ne "");
 	}
 	elsif ($key eq "image")
@@ -1243,8 +1251,11 @@ sub Info2Section {
 	    $line_ref = $self->UpdateSectionNameLine ($sectinfo{"name"}, $line_ref, $sectinfo{"original_name"});
 	    delete ($sectinfo{"name"});
 	}
+        elsif ($key eq "vga"){
+	  $line_ref->{"value"} = $sectinfo{"vgamode"} if (defined  $sectinfo{"vgamode"});
+        }
 	elsif ($key eq "image" || $key eq "initrd" || $key eq "root"
-	    || $key eq "vga" || $key eq "append" || $key eq "wildcard")
+	     || $key eq "append" || $key eq "wildcard")
 	{
 	    if ($type eq "chainloader" || ! defined ($sectinfo{$key}))
 	    {
@@ -1289,11 +1300,16 @@ sub Info2Section {
 	    || $key eq "wildcard" || $key eq "image" || $key eq "other")
 	{
 	    $key = "other" if ($key eq "chainloader");
+	    $key = "vga" if ($key eq "vgamode");
 	    push @lines, {
 		"key" => $key,
 		"value" => $value,
 	    };
 	}
+        else
+        {
+          $self->l_milestone ("Core::Info2Section: Unknown key $key with value $value");
+        }
     }
 
     my $ret = $self->FixSectionLineOrder (\@lines,
@@ -1328,7 +1344,6 @@ sub Global2Info {
 	if ($key eq "default" || $key eq "timeout" || $key eq "gfxmenu" || $key eq "password")
 	{
 	    $ret{$key} = $val;
-            $ret{$key} = $val*10 if ($key eq "timeout");
 	}
 	elsif ($key eq "boot")
 	{
@@ -1569,14 +1584,14 @@ sub ParseMenuFileLines {
     my @global = @{+shift @sects};
 
     @sects = map {
-	$self->l_debug ("Core::ParseMenuFileLines: section lines to convert :\n'" .
+	$self->l_milestone ("Core::ParseMenuFileLines: section lines to convert :\n'" .
 			join("'\n' ",
 			     map {
 				 $_->{"key"} . " => " . $_->{"value"};
 			     } @{$_}) . "'"
 			);
 	my $s = $self->Section2Info ($_);
-	$self->l_debug ("Core::ParseMenuFileLines: parsing result :\n'" .
+	$self->l_milestone ("Core::ParseMenuFileLines: parsing result :\n'" .
 			join("'\n' ",
 			     map {
 				 m/^__/ ? () : $_ . " => '" . $s->{$_} . "'";
@@ -1666,7 +1681,24 @@ sub UpdateBootloader {
     my $ok = 1;
     foreach my $file (@files) {
 	if ( -f "$file.new" ) {
-	    rename "$file", "$file.old" if -f "$file";
+            #backup file only if they exist and previos backup is enought old
+            # 6 hours
+            if (-f "$file")
+            {
+              if (-f "$file.old")
+              {
+                my $mtime = time - ( stat("$file.old"))[9];
+                my $hours = $mtime / 3600;
+                if ($hours > 6) 
+                {
+	          rename "$file", "$file.old";
+                }
+              }
+              else
+              {
+	        rename "$file", "$file.old";
+              }
+            }
 	    unless ( rename "$file.new",  "$file" ) {
 		$self->l_error ("Core::UpdateBootloader: Error occurred while updating file $file");
 		$ok = undef;
@@ -1676,17 +1708,6 @@ sub UpdateBootloader {
     return $ok;
 }
 
-#
-# map<string,any> GetMetaData ()
-# sub GetMetaData() {
-#     return undef;
-# }
-#
-# boolean InitializeBootloader ()
-# sub InitializeBootloader {
-#     return undef;
-# }
-
 =item
 C<< $status = Bootloader::Core->RunCommand ($command, $log_file); >>
 
@@ -1705,14 +1726,19 @@ sub RunCommand {
 
     if (defined ($log))
     {
-	unlink "$log" if -f $log;
-	$command = "$command >>$log 2>&1";
+	$command = "$command >$log 2>&1";
     }
     else
     {
 	$command = "$command >/dev/null 2>/dev/null";
     }
-    return system ($command);
+    my $ret = system ($command);
+
+    my $output = `cat $log`;
+    $self->l_milestone("run $command - ret $ret + output: $output");
+    $self->l_error("Command '$command' failed with code $ret and output: $output") unless $ret == 0;
+
+    return $ret;
 }
 
 =item
