@@ -335,14 +335,14 @@ sub UnixDev2GrubDev {
     # This gives me the devicename, whether $dev is the device or a link!
     # This works for udev (kernel) devices only, devicemapper doesn't 
     # need to be changed here 
+    my $original = $dev;
     my $kernel_dev = $self->GetKernelDevice($dev);
-    my $original = $kernel_dev;
 
     $self->l_milestone ("GRUB::UnixDev2GrubDev: kernel device: $kernel_dev");
 
     my $partition = undef;
-    if ($dev =~ m#/dev/md\d+#) {
-	my @members = @{$self->MD2Members ($dev) || []};
+    if ($kernel_dev =~ m#/dev/md\d+#) {
+	my @members = @{$self->MD2Members ($kernel_dev) || []};
 	# FIXME! This only works for mirroring (Raid1)
 	$kernel_dev = $members[0] || $kernel_dev;
 	$self->l_milestone ("GRUB::UnixDev2GrubDev: First device of MDRaid: $original --> $kernel_dev");
@@ -512,8 +512,8 @@ sub UnixPath2GrubPath {
 	$dev = "";
     }
 
-    $path = $dev . $path;
-    $self->l_debug ("GRUB::UnixPath2GrubPath: Translated path: $orig_path, prefix $preset_dev, to: $path");
+    $path = $dev . ($path||"");
+    $self->l_milestone ("GRUB::UnixPath2GrubPath: Translated path: $orig_path, prefix $preset_dev, to: $path");
     return $path;
 }
 
@@ -612,14 +612,14 @@ sub ParseLines {
 	{
           #multipath handling, multipath need real device, because multipath
           # device have broken geometry (bnc #448110)
-          if (defined $self->{"multipath"} && defined $self->{"multipath"}->{$2}){
-            $devmap{ $self->{"multipath"}->{$2} } = $1;
+          if (defined $self->{"multipath"} && defined $self->{"multipath"}->{$self->GetKernelDevice($2)}){
+            $devmap{ $self->{"multipath"}->{$self->GetKernelDevice($2)} } = $1;
           } else {
-	    $devmap{$2} = $1;
+      	    $devmap{$2} = $1;
           }
 	}
     };
-    $self->l_debug ("GRUB::Parselines: avoided_reading device map.") if (! $avoid_reading_device_map );
+    $self->l_milestone ("GRUB::Parselines: avoided_reading device map.") if ($avoid_reading_device_map );
     $self->{"device_map"} = \%devmap	if (! $avoid_reading_device_map);
 
     # and now proceed with menu.lst
@@ -811,6 +811,8 @@ sub CreateGrubConfLines() {
     my %glob = %{$self->{"global"}};
     my @grub_conf = ();
     my %s1_devices = ();
+    # md_discs stores grub discs which should be synchronized (needed for correct stage2 location)
+    my $md_discs = {};
 
     {
 	my $dev;
@@ -818,7 +820,7 @@ sub CreateGrubConfLines() {
 
 	# boot_custom => "selectdevice:Custom Boot Partition::"
 	$dev = delete($glob{"boot_custom"});
-	$s1_devices{$dev} = 1 if defined $dev;
+	$s1_devices{$dev} = 1 if (defined $dev and $dev ne "");
 	
 	# boot_mbr    => "bool:Boot from Master Boot Record:false",
 	$flag = delete $glob{"boot_mbr"};
@@ -830,13 +832,16 @@ sub CreateGrubConfLines() {
 	}
 
 	# boot_md_mbr   synchronize mbr of disc in md raid 
-        #(it is little tricky as md raid synchronize only partitions)
+  #(it is little tricky as md raid synchronize only partitions)
 	$flag = delete $glob{"boot_md_mbr"};
 	if (defined $flag and $flag ne "") {
             my @discs = split(/,/,$flag);
             chomp @discs;
             foreach my $mbr_disc (@discs){
-	      $s1_devices{$mbr_disc} = 1;
+      	      $s1_devices{$mbr_disc} = 1;
+              my $gdev = $self->UnixDev2GrubDev($mbr_disc);
+              $md_discs->{$gdev} = substr($gdev,1,-1);
+	            $self->l_milestone ("GRUB::CreateGrubConfLines: md_mbr device: $gdev ");
             }
 	}
 
@@ -960,7 +965,17 @@ sub CreateGrubConfLines() {
 	{
 	    my $options = join " ", @{$grub_conf_item->{"options"} || []};
 	    my $location = $self->UnixDev2GrubDev ($grub_conf_item->{"device"});
-	    my $line = "setup $options $location " . $stage1dev || $location;
+      my $stage_location = $stage1dev || $location;
+      if ($md_discs->{$location})
+      {
+        $self->l_milestone("GrubConfCreate: detected md_discs $location");
+        $stage_location =~ m/\(([^,)]+)(,[^)]+)?\)/;
+        my $old_disc = $1;
+        my $new_disc = $md_discs->{$location};
+        $stage_location =~ s/$old_disc/$new_disc/;
+        $self->l_milestone("GrubConfCreate: resulting location $stage_location");
+      }
+	    my $line = "setup $options $location " . $stage_location; 
 	    push @grub_conf, $line;
 	    delete $s1_devices{$grub_conf_item->{"device"}};
 	}
@@ -1069,13 +1084,13 @@ sub Section2Info {
 		    if (exists $ret{"xen_append"}) {
 		        my $xen_append = $ret{"xen_append"};
 		        while ($xen_append =~
-			    s/(.*)console=(\S+)\s*(.*)$/$1$3/o) {
-		              my $del_console = $2;
-				$xen_append =~
-			        s/(.*)${del_console}=\w+\s*(.*)$/$1$2/g;
-			}
-                        $xen_append =~ s/\s*(\S*)\s*$/$1/;
-			$ret{"xen_append"} = "console=$console $speed $xen_append";
+			            s/(.*)console=(\S+)\s*(.*)$/$1$3/o) {
+		           my $del_console = $2;
+      				$xen_append =~
+			          s/(.*)${del_console}=\w+\s*(.*)$/$1$2/g;
+			      }
+            $xen_append =~ s/\s*(\S*)\s*$/$1/;
+      			$ret{"xen_append"} = "console=$console $speed $xen_append";
 		    } else {
 		        $ret{"xen_append"} = "console=$console $speed";
 		    }
@@ -1085,6 +1100,11 @@ sub Section2Info {
 	}
 	elsif ($key eq "xen")
 	{
+	    if ($val =~ /^(?:(.*)\s+)?vga=mode-(\S+)(?:\s+(.*))?$/)
+	    {
+		    $ret{"vgamode"} = $2 if $2 ne "";
+    		$val = $self->MergeIfDefined ($1, $3);
+	    }
 	    # split into loader and parameter, note that the regex does
 	    # always match
 	    $val =~ /^\s*(\S+)(?:\s+(.*))?$/;
@@ -1447,9 +1467,11 @@ sub Info2Section {
         }
 	elsif ($key eq "kernel") {
 	    if ($type eq "xen") {
-		$line_ref->{"value"} =
-		    $self->UnixPath2GrubPath (delete($sectinfo{"xen"}), $grub_root)
-		    . " " . (delete($sectinfo{"xen_append"}) || "");
+        my $xen = $self->UnixPath2GrubPath (delete($sectinfo{"xen"}), $grub_root);
+        my $append = (delete($sectinfo{"xen_append"}) || "");
+        my $vga = delete ($sectinfo{"vgamode"}) || "";
+        $vga = "vga=mode-$vga " if $vga ne "";
+        $line_ref->{"value"} = "$xen $vga$append";
 	    }
 	    elsif ($type eq "image") {
 		$line_ref->{"value"} = $self->CreateKernelLine (\%sectinfo, $grub_root);
@@ -1530,8 +1552,11 @@ sub Info2Section {
 
     # keep a hard order for the following three entries
     if (exists $sectinfo{"xen"} && $type eq "xen") {
-      my $value = $self->UnixPath2GrubPath ($sectinfo{"xen"}, $grub_root)
-                      . " " . ($sectinfo{"xen_append"} || "");
+      my $xen = $self->UnixPath2GrubPath (delete($sectinfo{"xen"}), $grub_root);
+      my $append = (delete($sectinfo{"xen_append"}) || "");
+      my $vga = $sectinfo{"vgamode"} || "";
+      $vga = "vgamode=$vga " if $vga ne "";
+      my $value = "$xen $vga$append";
       push @lines, {
 	"key" => "kernel",
         "value" => $value,
@@ -1657,7 +1682,7 @@ sub Global2Info {
 	    next;
 	}
 
-	my ($type) = split(/:/, $go->{$key}||"");
+	my $type = $go->{$key}||"";
 
 	if ($key eq "root" or $key eq "rootnoverify") {
 	    $grub_root = $val;
@@ -1820,7 +1845,7 @@ sub Info2Global {
 	# only accept known global options :-)
 	next unless exists $go->{$key};
 	next if $key =~ m/^boot_/; # handled else where
-	my ($type) = split /:/, $go->{$key};
+	my $type = $go->{$key} || "";
 
 	if ($key eq "default") {
 	    $value = $self->IndexOfSection ($value, $sections_ref) || 0;
@@ -1971,9 +1996,9 @@ sub GrubDev2MountPoint {
 	$self->l_debug ("GRUB::GrubDev2MountPoint : MD Array: $md ; Members: $members");
 
 	foreach my $md_member (@{$members_ref}) {
-	    if ($device eq $md_member) {
+	    if ($self->GetKernelDevice($device) eq $self->GetKernelDevice($md_member)) {
 	        $self->l_milestone ("GRUB::GrubDev2MountPoint : find md: $md");
-		push (@devices, $md);
+      		push (@devices, $md);
 	    }
 	}
     }
@@ -1984,9 +2009,9 @@ sub GrubDev2MountPoint {
     foreach $device (@devices) {
 	while ((my $mp, my $d) = each (%{$self->{"mountpoints"}})) {
 	    $self->l_debug ("GRUB::GrubDev2MountPoint : record $mp <-> $d");
-	    if ($self->GetKernelDevice($d) eq $device) { 
-	        $self->l_milestone ("GRUB::GrubDev2MountPoint : find mountpoint: $mp");
-		$mountpoint = $mp;
+	    if ($self->GetKernelDevice($d) eq $self->GetKernelDevice($device)) { 
+	      $self->l_milestone ("GRUB::GrubDev2MountPoint : find mountpoint: $mp");
+    		$mountpoint = $mp;
 	    }
 	}
     }
