@@ -140,7 +140,7 @@ sub DumpLog {
 
 	    # If writing to perl logfile, also print warnings to STDERR
 	    if ($using_logfile) {
-		print STDERR (timestamp() . " WARNING: $message\n");
+		print STDERR ("Perl-Bootloader: ".timestamp() . " WARNING: $message\n");
 	    }
 	}
 	elsif ($level eq "error")
@@ -149,7 +149,7 @@ sub DumpLog {
 
 	    # If writing to perl logfile, also print errors to STDERR
 	    if ($using_logfile) {
-		print STDERR (timestamp() . " ERROR: $message\n");
+		print STDERR ("Perl-Bootloader: ".timestamp() . " ERROR: $message\n");
 	    }
 	}
 	else
@@ -203,9 +203,9 @@ sub ReadMountPoints {
 	    {
 		if ($dev =~ m/^LABEL=(.*)/)
 		{
-                    $dev = $udevmap->{"/dev/disk/by-label/$1"};
+                    $dev = "/dev/disk/by-label/$1"; #do not translate otherwise it changes root always bnc#575362
                 } elsif ($dev =~ m/UUID=(.*)/){
-                    $dev = $udevmap->{"/dev/disk/by-uuid/$1"};
+                    $dev = "/dev/disk/by-uuid/$1";
                 }
                 $mp =~ s/\\040/ /; #handle spaces in fstab
 		$mountpoints{$mp} = $dev;
@@ -241,7 +241,7 @@ sub ReadPartitions {
 
     # get disk devices
     my @disks = grep {
-	!m/^\./ and -r "$sb/$_/range" and qx{ cat $sb/$_/range } > 1
+	!m/^\./ && (( -r "$sb/$_/ext_range" and qx{ cat $sb/$_/ext_range } > 1) || ( -r "$sb/$_/range" and qx{ cat $sb/$_/range } > 1))
     } readdir(BLOCK_DEVICES);
     closedir BLOCK_DEVICES;
 
@@ -273,13 +273,15 @@ sub ReadPartitions {
             $logger->milestone("ReadPartitions: Finded parts: ". join (",",@parts));
 
 	    # generate proper device names and other info for all @part[ition]s
+      #raid have ! in names for /dev/raid/name (bnc#607852)
+      $dev_disk =~ s:!:/:;
 	    foreach my $part (@parts)
 	    {
 	        chomp ($part);
 	        $part = "/dev/$part";
-                #raid have ! in names for /dev/raid/name
-                $part =~ s:!:/:;
-                $part = $udevmap->{$part} if (defined $udevmap->{$part});
+          #raid have ! in names for /dev/raid/name
+          $part =~ s:!:/:;
+          $part = $udevmap->{$part} if (defined $udevmap->{$part});
 
 	        my $index = substr ($part, length ($dev_disk));
 	        while (length ($index) > 0 && substr ($index, 0, 1) !~ /[0-9]/)
@@ -658,17 +660,16 @@ sub ReadRAID1Arrays {
     $mdadm = AddPathToExecutable("mdadm");
 
     if (-e $mdadm) {
-	open (MD, "$mdadm --detail --verbose --scan |");
+      open (MD, "$mdadm --detail --verbose --scan |");
     }
     else {
-	$logger->error ("The command \"mdadm\" is not available.");
-	$logger->error ("Is the package \"mdadm\" installed?");
+    	$logger->milestone ("The command \"mdadm\" is not available.");
+    	$logger->milestone ("Expect that system doesn't have MD array.");
 
-	# If the command "mdadm" isn't available, return a reference to an
-	# empty hash
-	return \%mapping;
+    	# If the command "mdadm" isn't available, return a reference to an
+    	# empty hash
+    	return \%mapping;
     }
-    
 
     my ($array, $level, $num_devices);
     $logger->milestone("Tools::ReadRAID1Arrays: start parsing mdadm --detail --verbose --scan:");
@@ -689,7 +690,10 @@ sub ReadRAID1Arrays {
             # we could test $num_device against number of found devices to
             # detect degradedmode but that does not matter here (really?) 
              $logger->milestone("Tools::ReadRAID1Arrays: set to array $array values $1");
-             $mapping{$array} = [ split(/,/, $1) ];
+      	     my @devices = split(/,/,$1);
+	           if ($devices[0]=~ m/^.*\d+$/){ #add only non-disc arrays
+      	       $mapping{$array} = [ @devices ];
+      	     }
         }
     }
     $logger->milestone("Tools::ReadRAID1Arrays: finish parsing mdadm --detail --verbose --scan:");
@@ -862,7 +866,9 @@ bootloader to use the current configuration
 
 
 sub UpdateBootloader {
-    $lib_ref->UpdateBootloader ();
+    my $ret = $lib_ref->UpdateBootloader ();
+    DumpLog ($lib_ref->{"loader"});
+    return $ret;
 }
 
 
@@ -1125,14 +1131,17 @@ sub AddSection {
 	chomp ($arch);
 
 	if ($arch eq "i386") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume nosmp noapic maxcpus=0 edd=off x11failsafe";
+	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume nosmp noapic maxcpus=0 edd=off x11failsafe nomodeset";
 	}
 	elsif ($arch eq "x86_64") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume edd=off x11failsafe";
+	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume edd=off x11failsafe nomodeset";
 	}
 	elsif ($arch eq "ia64") {
 	    $new{"append"} = "ide=nodma nohalt noresume 3";
 	}
+	elsif ($arch eq "s390x") {
+	    $new{"append"} = "";
+  }
 	else {
 	    print ("Architecture $arch does not support failsafe entries.\n");
 	}
