@@ -12,11 +12,16 @@ BEGIN {
         getContDmraidCoInfo => ["function", "integer", "any", "&string", "any", "any"],
         getDmmultipathCoInfo => ["function", "integer", "any", "&string", "any"],
         getContDmmultipathCoInfo => ["function", "integer", "any", "&string", "any", "any"],
+        getMdPartCoInfo => ["function", "integer", "any", "&string", "any"],
+        getContMdPartCoInfo => ["function", "integer", "any", "&string", "any", "any"],
+        setImsmDriver => ["function", "void", "any", "integer"],
+        getImsmDriver => ["function", "integer", "any"],
         getVolumes => ["function", "void", "any", ["&list", "any"]],
         getVolume => ["function", "integer", "any", "&string", "any"],
         getPartitionInfo => ["function", "integer", "any", "&string", ["&list", "any"]],
         getLvmLvInfo => ["function", "integer", "any", "&string", ["&list", "any"]],
         getMdInfo => ["function", "integer", "any", ["&list", "any"]],
+        getMdPartInfo => ["function", "integer", "any", "&string", ["&list", "any"]],
         getNfsInfo => ["function", "integer", "any", ["&list", "any"]],
         getLoopInfo => ["function", "integer", "any", ["&list", "any"]],
         getDmInfo => ["function", "integer", "any", ["&list", "any"]],
@@ -29,7 +34,7 @@ BEGIN {
         resizePartition => ["function", "integer", "any", "&string", "integer"],
         resizePartitionNoFs => ["function", "integer", "any", "&string", "integer"],
         updatePartitionArea => ["function", "integer", "any", "&string", "integer", "integer"],
-        freeCylindersAfterPartition => ["function", "integer", "any", "&string", "&integer"],
+        freeCylindersAroundPartition => ["function", "integer", "any", "&string", "&integer", "&integer"],
         nextFreePartition => ["function", "integer", "any", "&string", "integer", "&integer", "&string"],
         createPartitionKb => ["function", "integer", "any", "&string", "integer", "integer", "integer", "&string"],
         createPartitionAny => ["function", "integer", "any", "&string", "integer", "&string"],
@@ -44,8 +49,7 @@ BEGIN {
         getUnusedPartitionSlots => ["function", "integer", "any", "&string", ["&list", "any"]],
         destroyPartitionTable => ["function", "integer", "any", "&string", "&string"],
         initializeDisk => ["function", "integer", "any", "&string", "boolean"],
-        defaultDiskLabel => ["function", "string", "any"],
-        defaultDiskLabelSize => ["function", "string", "any", "integer"],
+        defaultDiskLabel => ["function", "string", "any", "&string"],
         changeFormatVolume => ["function", "integer", "any", "&string", "boolean", "integer"],
         changeLabelVolume => ["function", "integer", "any", "&string", "&string"],
         changeMkfsOptVolume => ["function", "integer", "any", "&string", "&string"],
@@ -61,6 +65,8 @@ BEGIN {
         setCryptPassword => ["function", "integer", "any", "&string", "&string"],
         forgetCryptPassword => ["function", "integer", "any", "&string"],
         getCryptPassword => ["function", "integer", "any", "&string", "&string"],
+        verifyCryptPassword => ["function", "integer", "any", "&string", "&string", "boolean"],
+        needCryptPassword => ["function", "boolean", "any", "&string"],
         setCrypt => ["function", "integer", "any", "&string", "boolean"],
         setCryptType => ["function", "integer", "any", "&string", "boolean", "integer"],
         getCrypt => ["function", "integer", "any", "&string", "&boolean"],
@@ -73,8 +79,11 @@ BEGIN {
         forgetResizeVolume => ["function", "integer", "any", "&string"],
         setRecursiveRemoval => ["function", "void", "any", "boolean"],
         getRecursiveRemoval => ["function", "boolean", "any"],
+        getRecursiveUsing => ["function", "integer", "any", "&string", ["&list", "string"]],
         setZeroNewPartitions => ["function", "void", "any", "boolean"],
         getZeroNewPartitions => ["function", "boolean", "any"],
+        setPartitionAlignment => ["function", "void", "any", "integer"],
+        getPartitionAlignment => ["function", "integer", "any"],
         setDefaultMountBy => ["function", "void", "any", "integer"],
         getDefaultMountBy => ["function", "integer", "any"],
         setDefaultFs => ["function", "void", "any", "integer"],
@@ -109,9 +118,11 @@ BEGIN {
         changeMdParity => ["function", "integer", "any", "&string", "integer"],
         checkMd => ["function", "integer", "any", "&string"],
         getMdStateInfo => ["function", "integer", "any", "&string", "any"],
-        computeMdSize => ["function", "integer", "any", "integer", ["list", "string"], "&integer"],
-        addNfsDevice => ["function", "integer", "any", "&string", "&string", "integer", "&string"],
-        checkNfsDevice => ["function", "integer", "any", "&string", "&string", "&integer"],
+        getMdPartCoStateInfo => ["function", "integer", "any", "&string", "any"],
+        computeMdSize => ["function", "integer", "any", "integer", ["&list", "string"], "&integer"],
+        removeMdPartCo => ["function", "integer", "any", "&string", "boolean"],
+        addNfsDevice => ["function", "integer", "any", "&string", "&string", "integer", "&string", "boolean"],
+        checkNfsDevice => ["function", "integer", "any", "&string", "&string", "boolean", "&integer"],
         createFileLoop => ["function", "integer", "any", "&string", "boolean", "integer", "&string", "&string", "&string"],
         modifyFileLoop => ["function", "integer", "any", "&string", "&string", "boolean", "integer"],
         removeFileLoop => ["function", "integer", "any", "&string", "boolean"],
@@ -122,6 +133,7 @@ BEGIN {
         setCacheChanges => ["function", "void", "any", "boolean"],
         isCacheChanges => ["function", "boolean", "any"],
         commit => ["function", "integer", "any"],
+        getErrorString => ["function", "string", "any", "integer"],
         createBackupState => ["function", "integer", "any", "&string"],
         restoreBackupState => ["function", "integer", "any", "&string"],
         checkBackupState => ["function", "boolean", "any", "&string"],
@@ -130,16 +142,19 @@ BEGIN {
         checkDeviceMounted => ["function", "boolean", "any", "&string", ["&list", "string"]],
         umountDevice => ["function", "boolean", "any", "&string"],
         mountDevice => ["function", "boolean", "any", "&string", "&string"],
+        activateEncryption => ["function", "integer", "any", "&string", "boolean"],
         mountDeviceOpts => ["function", "boolean", "any", "&string", "&string", "&string"],
         mountDeviceRo => ["function", "boolean", "any", "&string", "&string", "&string"],
         checkDmMapsTo => ["function", "boolean", "any", "&string"],
         removeDmTableTo => ["function", "void", "any", "&string"],
-        getFreeInfo => ["function", "boolean", "any", "&string", "&integer", "&integer", "&integer", "&boolean", "&boolean", "boolean"],
+        getFreeInfo => ["function", "boolean", "any", "&string", "boolean", "any", "boolean", "any", "boolean"],
         readFstab => ["function", "boolean", "any", "&string", ["&list", "any"]],
         activateHld => ["function", "void", "any", "boolean"],
         activateMultipath => ["function", "void", "any", "boolean"],
         rescanEverything => ["function", "void", "any"],
+        rescanCryptedObjects => ["function", "boolean", "any"],
         dumpObjectList => ["function", "void", "any"],
+        dumpCommitInfos => ["function", "void", "any"],
         getContVolInfo => ["function", "integer", "any", "&string", "any"],
     );
 }
